==> audiochain-1.1.5-src.tgz: org/audiochain/ui/gui/mixer/CreateNewAudioProjectDialog.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui.mixer;

import static org.audiochain.model.ObjectStore.*;
import static org.audiochain.ui.gui.Dialog.*;
import static org.audiochain.ui.gui.GlobalGuiContext.*;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;

import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.audiochain.model.AudioProject;
import org.audiochain.ui.gui.AudioProjectFileFilter;
import org.audiochain.ui.gui.Dialog;
import org.audiochain.ui.gui.ImageButtonComponent;
import org.audiochain.ui.gui.NotificationDialog;

public class CreateNewAudioProjectDialog extends NotificationDialog {
	JLabel projectNameLabel;
	JTextField projectNameField;
	JLabel projectPathLabel;
	JTextField projectPathField;
	JComponent searchProjectPathButton;
	JLabel templatePathLabel;
	JTextField templatePathField;
	JComponent searchTemplatePathButton;
	AudioProject audioProject;
	JPanel mainPanel;

	public CreateNewAudioProjectDialog(Object caller) {
		super(caller);
		init();
	}

	protected void init() {
		createMainComponents();
		createApplyButton("Create Audio Project");
		createProjectNameComponents();
		createProjectPathComponents("Project Path:");
		createTemplatePathComponents();
		evaluateMandatoryFields();

		createLayout();

		if (isButtonPressed(caller, "Create New Audio Project", mainPanel, applyButton)) {
			String projectPath = projectPathField.getText();
			if (projectPath == null || projectPath.trim().isEmpty()) {
				throw new IllegalStateException("The audio project path must not be null.");
			}
			File projectPathFile = new File(projectPath);
			if (!projectPathFile.exists()) {
				if (!projectPathFile.mkdirs()) {
					Dialog.message(caller, "Error while creating new Project Directory", "The project directory '" + projectPath + "' can not be created.");
					return;
				}
			}
			getGuiSettings().setLastNewProjectPathName(projectPath);

			String templateFileName = templatePathField.getText();
			AudioProject templateProject = null;
			if (templateFileName != null && !templateFileName.trim().isEmpty()) {
				try {
					templateProject = AudioProject.load(new File(templateFileName));
				} catch (Exception ex) {
					ex.printStackTrace();
					Dialog.message(caller, "Error while loading project template", ex);
				}
			}

			try {
				AudioProject newProject = AudioProject.createAudioProject(projectPath, templateProject);
				String projectName = projectNameField.getText();
				if (projectName != null && !projectName.trim().isEmpty()) {
					newProject.setName(projectName);
				}
				if (templateFileName != null && !templateFileName.trim().isEmpty()) {
					newProject.setTemplateFileName(templateFileName);
				}
				getGuiSettings().setProjectTemplateFileName(templateFileName);
				this.audioProject = newProject;
			} catch (Exception ex) {
				ex.printStackTrace();
				Dialog.message(caller, "Error while creating audio project", ex);
			}
		}
	}

	protected void evaluateMandatoryFields() {
		String projectPath = projectPathField.getText();
		if (projectPath != null && !projectPath.trim().isEmpty()) {
			evaluateProjectPath();
		}
		String templatePath = templatePathField.getText();
		if (templatePath != null && !templatePath.trim().isEmpty()) {
			evaluateTemplateFile();
		}
	}

	protected void createApplyButton(String text) {
		applyButton = new JButton(text);
		applyButton.setEnabled(false);
	}

	protected void createMainComponents() {
		mainPanel = new JPanel();
		mainPanel.setOpaque(false);
	}

	private void createLayout() {
		GroupLayout layout = new GroupLayout(mainPanel);
		mainPanel.setLayout(layout);
		layout.setAutoCreateContainerGaps(true);
		layout.setAutoCreateGaps(true);

		layout.setHorizontalGroup(layout
				.createParallelGroup()
				.addComponent(notificationLabel)
				.addGroup(
						layout.createSequentialGroup()
								.addGroup(layout.createParallelGroup().addComponent(projectNameLabel).addComponent(projectPathLabel).addComponent(templatePathLabel))
								.addGroup(
										layout.createParallelGroup().addComponent(projectNameField)
												.addGroup(layout.createSequentialGroup().addComponent(projectPathField).addComponent(searchProjectPathButton))
												.addGroup(layout.createSequentialGroup().addComponent(templatePathField).addComponent(searchTemplatePathButton)))));

		layout.setVerticalGroup(layout
				.createSequentialGroup()
				.addComponent(notificationLabel)
				.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(projectNameLabel).addComponent(projectNameField))
				.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(projectPathLabel).addComponent(projectPathField).addComponent(searchProjectPathButton))
				.addGroup(
						layout.createParallelGroup(Alignment.BASELINE).addComponent(templatePathLabel).addComponent(templatePathField).addComponent(projectPathField)
								.addComponent(searchTemplatePathButton))
				// add additional gap to the dialog buttons, because the
				// notification label causes an additional distance to the
				// dialog title on the top
				.addGap(20));
	}

	protected void createTemplatePathComponents() {
		templatePathLabel = new JLabel("Template File:");
		templatePathField = new JTextField(getGuiSettings().getProjectTemplateFileName(), 30);
		templatePathField.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void insertUpdate(DocumentEvent e) {
				evaluateTemplateFile();
			}

			@Override
			public void removeUpdate(DocumentEvent e) {
				evaluateTemplateFile();
			}

			@Override
			public void changedUpdate(DocumentEvent e) {
				evaluateTemplateFile();
			}
		});
		searchTemplatePathButton = new ImageButtonComponent("org/audiochain/ui/gui/open-directory.png");
		searchTemplatePathButton.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				String fileName = templatePathField.getText();
				if (fileName == null || fileName.trim().isEmpty()) {
					fileName = getGuiSettings().getProjectTemplateFileName();
				}
				JFileChooser jfc = fileName != null && !fileName.trim().isEmpty() ? new JFileChooser(new File(fileName)) : new JFileChooser();
				jfc.setFileFilter(new AudioProjectFileFilter());
				int status = jfc.showOpenDialog(null);
				if (status == JFileChooser.APPROVE_OPTION) {
					File path = jfc.getSelectedFile();
					String newPath = path.getAbsolutePath();
					templatePathField.setText(newPath);
				}
			}
		});
	}

	protected void createProjectPathComponents(String label) {
		projectPathLabel = new JLabel(label);
		projectPathField = new JTextField(30);
		projectPathField.setText(getGuiSettings().getLastNewProjectPathName());
		projectPathField.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void insertUpdate(DocumentEvent e) {
				evaluateProjectPath();
			}

			@Override
			public void removeUpdate(DocumentEvent e) {
				evaluateProjectPath();
			}

			@Override
			public void changedUpdate(DocumentEvent e) {
				evaluateProjectPath();
			}
		});
		searchProjectPathButton = new ImageButtonComponent("org/audiochain/ui/gui/open-directory.png");
		searchProjectPathButton.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				File initialPath = null;
				String fileName = projectPathField.getText();
				if (fileName == null || fileName.trim().isEmpty()) {
					fileName = getGuiSettings().getLastNewProjectPathName();
					if (fileName != null) {
						initialPath = new File(fileName);
						File parentFile = initialPath.getParentFile();
						if (initialPath.exists() && parentFile != null) {
							File grandParentFile = parentFile.getParentFile();
							if (initialPath.isFile() && grandParentFile != null) {
								// open the directory where potentially other
								// projects may be located
								initialPath = grandParentFile;
							} else {
								initialPath = parentFile;
							}
						}

					}
				} else {
					initialPath = new File(fileName);
				}
				JFileChooser jfc = initialPath != null ? new JFileChooser(initialPath) : new JFileChooser();
				jfc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				int status = jfc.showOpenDialog(null);
				if (status == JFileChooser.APPROVE_OPTION) {
					File path = jfc.getSelectedFile();
					String newPath = path.getAbsolutePath();
					projectPathField.setText(newPath);
				}
			}
		});
	}

	protected void createProjectNameComponents() {
		projectNameLabel = new JLabel("Project Name:");
		projectNameField = new JTextField(30);
	}

	protected String checkProjectPath() {
		String projectPathName = projectPathField.getText();
		if (projectPathName == null || projectPathName.trim().isEmpty()) {
			return "Please select a Project Path!";
		} else {
			File projectPathFile = new File(projectPathName.trim());
			if (!projectPathFile.exists() && projectPathFile.getParentFile() == null) {
				return "The Project Path must be an absolute path.";
			} else if (projectPathFile.exists() && projectPathFile.isFile()) {
				return "The Project Path is no directory.";
			} else if (projectPathFile.exists() && !projectPathFile.canWrite()) {
				return "The Project Path is not writable.";
			} else if (!projectPathFile.exists() && !hasAnyWritableParentDirectory(projectPathFile)) {
				return "The Project Path can not be created because there is no writable parent directory.";
			} else {
				return null;
			}
		}
	}

	private String checkTemplateFile() {
		String templateFileName = templatePathField.getText();
		if (templateFileName != null && !templateFileName.trim().isEmpty()) {
			File templateFile = new File(templateFileName.trim());
			if (!templateFile.exists()) {
				return "The Template File does not exist.";
			} else if (!templateFile.isFile()) {
				return "The Template File is no file.";
			} else if (!templateFile.canRead()) {
				return "The Template File is not readable.";
			} else {
				try {
					AudioProject.load(templateFile);
				} catch (Exception e) {
					return "The Template File is corrupt. (" + e.getMessage().trim() + ")";
				}
			}
		}
		return null;
	}

	private void evaluateProjectPath() {
		putNotification(projectPathField, checkProjectPath());
	}

	private void evaluateTemplateFile() {
		putNotification(templatePathField, checkTemplateFile());
	}

	@Override
	protected boolean mandatoryFieldsValid() {
		return checkTemplateFile() == null;
	}

	public AudioProject getAudioProject() {
		return audioProject;
	}
}

==> audiochain-1.1.6-src.tgz: org/audiochain/ui/gui/mixer/CreateNewAudioProjectDialog.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui.mixer;

import static org.audiochain.model.ObjectStore.hasAnyWritableParentDirectory;
import static org.audiochain.ui.gui.Dialog.isButtonPressed;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;

import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.audiochain.model.AudioProject;
import org.audiochain.ui.gui.AudioProjectFileFilter;
import org.audiochain.ui.gui.Dialog;
import org.audiochain.ui.gui.GuiSettings;
import org.audiochain.ui.gui.ImageButtonComponent;
import org.audiochain.ui.gui.NotificationDialog;

public class CreateNewAudioProjectDialog extends NotificationDialog {
	JLabel projectNameLabel;
	JTextField projectNameField;
	JLabel projectPathLabel;
	JTextField projectPathField;
	JComponent searchProjectPathButton;
	JLabel templatePathLabel;
	JTextField templatePathField;
	JComponent searchTemplatePathButton;
	AudioProject audioProject;
	JPanel mainPanel;

	public CreateNewAudioProjectDialog(Object caller) {
		super(caller);
		init();
	}

	protected void init() {
		createMainComponents();
		createApplyButton("Create Audio Project");
		createProjectNameComponents();
		createProjectPathComponents("Project Path:");
		createTemplatePathComponents();
		evaluateMandatoryFields();

		createLayout();

		if (isButtonPressed(caller, "Create New Audio Project", mainPanel, applyButton)) {
			String projectPath = projectPathField.getText();
			if (projectPath == null || projectPath.trim().isEmpty()) {
				throw new IllegalStateException("The audio project path must not be null.");
			}
			File projectPathFile = new File(projectPath);
			if (!projectPathFile.exists()) {
				if (!projectPathFile.mkdirs()) {
					Dialog.message(caller, "Error while creating new Project Directory", "The project directory '" + projectPath + "' can not be created.");
					return;
				}
			}
			GuiSettings.getInstance().setLastNewProjectPathName(projectPath);

			String templateFileName = templatePathField.getText();
			AudioProject templateProject = null;
			if (templateFileName != null && !templateFileName.trim().isEmpty()) {
				try {
					templateProject = AudioProject.load(new File(templateFileName));
				} catch (Exception ex) {
					ex.printStackTrace();
					Dialog.message(caller, "Error while loading project template", ex);
				}
			}

			try {
				AudioProject newProject = AudioProject.createAudioProject(projectPath, templateProject);
				String projectName = projectNameField.getText();
				if (projectName != null && !projectName.trim().isEmpty()) {
					newProject.setName(projectName);
				}
				if (templateFileName != null && !templateFileName.trim().isEmpty()) {
					newProject.setTemplateFileName(templateFileName);
				}
				GuiSettings.getInstance().setProjectTemplateFileName(templateFileName);
				this.audioProject = newProject;
			} catch (Exception ex) {
				ex.printStackTrace();
				Dialog.message(caller, "Error while creating audio project", ex);
			}
		}
	}

	protected void evaluateMandatoryFields() {
		String projectPath = projectPathField.getText();
		if (projectPath != null && !projectPath.trim().isEmpty()) {
			evaluateProjectPath();
		}
		String templatePath = templatePathField.getText();
		if (templatePath != null && !templatePath.trim().isEmpty()) {
			evaluateTemplateFile();
		}
	}

	protected void createApplyButton(String text) {
		applyButton = new JButton(text);
		applyButton.setEnabled(false);
	}

	protected void createMainComponents() {
		mainPanel = new JPanel();
		mainPanel.setOpaque(false);
	}

	private void createLayout() {
		GroupLayout layout = new GroupLayout(mainPanel);
		mainPanel.setLayout(layout);
		layout.setAutoCreateContainerGaps(true);
		layout.setAutoCreateGaps(true);

		layout.setHorizontalGroup(layout
				.createParallelGroup()
				.addComponent(notificationLabel)
				.addGroup(
						layout.createSequentialGroup()
								.addGroup(layout.createParallelGroup().addComponent(projectNameLabel).addComponent(projectPathLabel).addComponent(templatePathLabel))
								.addGroup(
										layout.createParallelGroup().addComponent(projectNameField)
												.addGroup(layout.createSequentialGroup().addComponent(projectPathField).addComponent(searchProjectPathButton))
												.addGroup(layout.createSequentialGroup().addComponent(templatePathField).addComponent(searchTemplatePathButton)))));

		layout.setVerticalGroup(layout
				.createSequentialGroup()
				.addComponent(notificationLabel)
				.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(projectNameLabel).addComponent(projectNameField))
				.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(projectPathLabel).addComponent(projectPathField).addComponent(searchProjectPathButton))
				.addGroup(
						layout.createParallelGroup(Alignment.BASELINE).addComponent(templatePathLabel).addComponent(templatePathField).addComponent(projectPathField)
								.addComponent(searchTemplatePathButton))
				// add additional gap to the dialog buttons, because the
				// notification label causes an additional distance to the
				// dialog title on the top
				.addGap(20));
	}

	protected void createTemplatePathComponents() {
		templatePathLabel = new JLabel("Template File:");
		templatePathField = new JTextField(GuiSettings.getInstance().getProjectTemplateFileName(), 30);
		templatePathField.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void insertUpdate(DocumentEvent e) {
				evaluateTemplateFile();
			}

			@Override
			public void removeUpdate(DocumentEvent e) {
				evaluateTemplateFile();
			}

			@Override
			public void changedUpdate(DocumentEvent e) {
				evaluateTemplateFile();
			}
		});
		searchTemplatePathButton = new ImageButtonComponent("org/audiochain/ui/gui/open-directory.png");
		searchTemplatePathButton.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				String fileName = templatePathField.getText();
				if (fileName == null || fileName.trim().isEmpty()) {
					fileName = GuiSettings.getInstance().getProjectTemplateFileName();
				}
				JFileChooser jfc = fileName != null && !fileName.trim().isEmpty() ? new JFileChooser(new File(fileName)) : new JFileChooser();
				jfc.setFileFilter(new AudioProjectFileFilter());
				int status = jfc.showOpenDialog(null);
				if (status == JFileChooser.APPROVE_OPTION) {
					File path = jfc.getSelectedFile();
					String newPath = path.getAbsolutePath();
					templatePathField.setText(newPath);
				}
			}
		});
	}

	protected void createProjectPathComponents(String label) {
		projectPathLabel = new JLabel(label);
		projectPathField = new JTextField(30);
		projectPathField.setText(GuiSettings.getInstance().getLastNewProjectPathName());
		projectPathField.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void insertUpdate(DocumentEvent e) {
				evaluateProjectPath();
			}

			@Override
			public void removeUpdate(DocumentEvent e) {
				evaluateProjectPath();
			}

			@Override
			public void changedUpdate(DocumentEvent e) {
				evaluateProjectPath();
			}
		});
		searchProjectPathButton = new ImageButtonComponent("org/audiochain/ui/gui/open-directory.png");
		searchProjectPathButton.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				File initialPath = null;
				String fileName = projectPathField.getText();
				if (fileName == null || fileName.trim().isEmpty()) {
					fileName = GuiSettings.getInstance().getLastNewProjectPathName();
					if (fileName != null) {
						initialPath = new File(fileName);
						File parentFile = initialPath.getParentFile();
						if (initialPath.exists() && parentFile != null) {
							File grandParentFile = parentFile.getParentFile();
							if (initialPath.isFile() && grandParentFile != null) {
								// open the directory where potentially other
								// projects may be located
								initialPath = grandParentFile;
							} else {
								initialPath = parentFile;
							}
						}

					}
				} else {
					initialPath = new File(fileName);
				}
				JFileChooser jfc = initialPath != null ? new JFileChooser(initialPath) : new JFileChooser();
				jfc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				int status = jfc.showOpenDialog(null);
				if (status == JFileChooser.APPROVE_OPTION) {
					File path = jfc.getSelectedFile();
					String newPath = path.getAbsolutePath();
					projectPathField.setText(newPath);
				}
			}
		});
	}

	protected void createProjectNameComponents() {
		projectNameLabel = new JLabel("Project Name:");
		projectNameField = new JTextField(30);
	}

	protected String checkProjectPath() {
		String projectPathName = projectPathField.getText();
		if (projectPathName == null || projectPathName.trim().isEmpty()) {
			return "Please select a Project Path!";
		} else {
			File projectPathFile = new File(projectPathName.trim());
			if (!projectPathFile.exists() && projectPathFile.getParentFile() == null) {
				return "The Project Path must be an absolute path.";
			} else if (projectPathFile.exists() && projectPathFile.isFile()) {
				return "The Project Path is no directory.";
			} else if (projectPathFile.exists() && !projectPathFile.canWrite()) {
				return "The Project Path is not writable.";
			} else if (!projectPathFile.exists() && !hasAnyWritableParentDirectory(projectPathFile)) {
				return "The Project Path can not be created because there is no writable parent directory.";
			} else {
				return null;
			}
		}
	}

	private String checkTemplateFile() {
		String templateFileName = templatePathField.getText();
		if (templateFileName != null && !templateFileName.trim().isEmpty()) {
			File templateFile = new File(templateFileName.trim());
			if (!templateFile.exists()) {
				return "The Template File does not exist.";
			} else if (!templateFile.isFile()) {
				return "The Template File is no file.";
			} else if (!templateFile.canRead()) {
				return "The Template File is not readable.";
			} else {
				try {
					AudioProject.load(templateFile);
				} catch (Exception e) {
					return "The Template File is corrupt. (" + e.getMessage().trim() + ")";
				}
			}
		}
		return null;
	}

	private void evaluateProjectPath() {
		putNotification(projectPathField, checkProjectPath());
	}

	private void evaluateTemplateFile() {
		putNotification(templatePathField, checkTemplateFile());
	}

	@Override
	protected boolean mandatoryFieldsValid() {
		return checkTemplateFile() == null;
	}

	public AudioProject getAudioProject() {
		return audioProject;
	}
}
